@@ -823,7 +823,7 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    shout2send,
+    shout2,
     "Sends data to an icecast server using libshout2",
     plugin_init,
     VERSION, "LGPL", "libshout2", "http://www.icecast.org/download/")
